/***************************************************************************
                          message.h  -  description
                             -------------------
    begin                : Wed Nov 28 2001
    copyright            : (C) 2001, 2002 by Thomas Friedrichsmeier
    email                : Thomas.Friedrichsmeier@ruhr-uni-bochum.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MESSAGE_H
#define MESSAGE_H

#include <timedobject.h>

#include "tpbase.h"

/**Container for two QCanvas Text-Objects, that are used to display
a message (like "you crashed").
  *@author Thomas Friedrichsmeier
  */

class QString;
class QCanvasSprite;
class CachablePixmapArray;
class TimeManager;

class Message : public TimedObject, public TPBase {
public:
	Message();
	~Message();
/** Displays the message string message. And a hint to press the key described
	by ack_key. Usually, the message is stopped, when it's bottom half is vertically
	centered in the view. If below is given, it stops that much below the half.
	If width is given, the message shown will not be wider than the given value */
	void displayMessage (QString & message, QString & ack_key, int below=0, int max_width=0);
/** Kills any message that might still be acitve (called when game gets aborted) */
	void kill ();
/** Message was acknowledged */
	void acknowledged();
private:
	QCanvasSprite * message_sprite;
	QCanvasSprite * ack_sprite;
	CachablePixmapArray	*message_pm;
	CachablePixmapArray	*ack_pm;
/** Phase of message-display: 0: Message coming up; 1: Message in place, Acknowledge-Text coming up;
	2: Both texts in place; 3: Message acknowledged, both texts scrolling out of view; 4: done, sprites got deleted */
	int phase;
	int text_dest_mid, text_dest_top;
	double message_speed, ack_speed;
//	int old_chunk_size;
/** Deletes the message- and ack- sprites and pixmaps (also kills timer and sets phase to 4) */
	void deleteSprites ();
	TimeManager *mtm;
protected:
	void timerTick (int id);
};

#endif
