/***************************************************************************
                          missionselector  -  description
                             -------------------
    begin                : Fri Jan 30 2004
    copyright            : (C) 2004, 2005, 2007 by Thomas Friedrichsmeier
    email                : tfry@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "missionselector.h"

#include <qlayout.h>
#include <qlabel.h>
#include <qimage.h>
#include <qcheckbox.h>
#include <qpushbutton.h>
#include <qlistview.h>
#include <qtextedit.h>
#include <qstringlist.h>
#include <qtabwidget.h>

#include <klocale.h>
#include <kfiledialog.h>

#include "taxipilot.h"
#include "cdp.h"

MissionSelector::MissionSelector (bool scan_missions, Face show_face, bool force_selection) : KDialogBase (tp) {
	prevent_close = force_selection;

	QWidget *page = new QWidget (this);
	setMainWidget (page);
	QVBoxLayout *mainvbox = new QVBoxLayout (page, 0, spacingHint ());

	tabbook = new QTabWidget (page);
	
	mwidget = new MissionWidget (tabbook, this);
	tabbook->addTab (mwidget, i18n ("Select Mission"));
	swidget = new SavesWidget (tabbook, this);
	tabbook->addTab (swidget, i18n ("Load saved game"));

	mainvbox->addWidget (tabbook);
	if (show_face == Saves) {
		tabbook->showPage (swidget);
	}
	connect (tabbook, SIGNAL (currentChanged (QWidget *)), this, SLOT (pageChanged (QWidget *)));

	mainvbox->addSpacing (10);
	select_on_startup_box = new QCheckBox (i18n ("Always show this dialog on startup"), page);
	select_on_startup_box->setChecked (cdp->selectMissionOnStartup());
	mainvbox->addWidget (select_on_startup_box);
	mainvbox->addSpacing (10);

	setCaption (i18n ("Chose mission to play"));

	if (scan_missions) {
		QString dummy = cdp->defaultPath();
		QStringList list = cdp->scanNewMissions (dummy);
		for (QStringList::Iterator it = list.begin(); it != list.end(); ++it) {
			mwidget->addMission (*it);
		}
	}
	
	mwidget->initialize ();
	swidget->initialize ();
	
	setButtonOK (KGuiItem (i18n ("Start mission")));
	showButtonApply (false);
	showButtonCancel (false);
}

MissionSelector::~MissionSelector() {
}

void MissionSelector::slotOk () {
	if (tabbook->currentPage () == mwidget) {
		mwidget->apply ();
	} else {
		swidget->apply ();
	}
	prevent_close = false;

	KDialogBase::slotOk ();
}

void MissionSelector::pageChanged (QWidget *widget) {
	if (widget == mwidget) {
		mwidget->selectionChanged ();
	} else {
		swidget->selectionChanged ();
	}
}

void MissionSelector::selectionChanged (Face in_face, bool valid) {
	if ((in_face == Mission) && (tabbook->currentPage () == mwidget)) {
		enableButtonOK (valid);
		return;
	} else if ((in_face == Saves) && (tabbook->currentPage () == swidget)){
		enableButtonOK (valid);
		return;
	}
	
	enableButtonOK (false);
}

void MissionSelector::closeEvent (QCloseEvent *e) {
	if (prevent_close) {
		e->ignore ();
		return;
	}

	KDialogBase::closeEvent (e);
}

// ########### BEGIN MissionWidget ####################


MissionWidget::MissionWidget (QWidget *parent, MissionSelector *m_sel) : QWidget (parent) {
	MissionWidget::parent = m_sel;

	QGridLayout *mainlayout = new QGridLayout (this);
	QVBoxLayout *mainvbox = new QVBoxLayout (mainlayout);
	mainvbox->setMargin (5);

	missionlist = new QListView (this);
	missionlist->addColumn (i18n ("Title"));
	missionlist->addColumn (i18n ("Author(s)"));
	missionlist->addColumn (i18n ("Location"));
	missionlist->setColumnWidth (2, 100);
	missionlist->setColumnWidthMode (2, QListView::Manual);
// these two columns are just internal containers not to be user-visible
	missionlist->addColumn ("splash_image");
	missionlist->hideColumn (3);
	missionlist->setColumnWidthMode (3, QListView::Manual);
	missionlist->addColumn ("abstract");
	missionlist->hideColumn (4);
	missionlist->setColumnWidthMode (4, QListView::Manual);

	missionlist->setSelectionMode (QListView::Single);
	connect (missionlist, SIGNAL (selectionChanged ()), this, SLOT (selectionChanged ()));
	mainvbox->addWidget (missionlist);

	QHBoxLayout *selectionbuttonbox = new QHBoxLayout (mainvbox);
	QPushButton *scanbutton = new QPushButton (i18n ("(Re-)scan directory for new missions"), this);
	connect (scanbutton, SIGNAL (clicked ()), this, SLOT (scanButtonPressed ()));
	QPushButton *addbutton = new QPushButton (i18n ("Add mission(s) manually"), this);
	connect (addbutton, SIGNAL (clicked ()), this, SLOT (addButtonPressed ()));
	delbutton = new QPushButton (i18n ("Remove selected mission from list"), this);
	connect (delbutton, SIGNAL (clicked ()), this, SLOT (delButtonPressed ()));
	selectionbuttonbox->addWidget (scanbutton);
	selectionbuttonbox->addStretch ();
	selectionbuttonbox->addWidget (addbutton);
	selectionbuttonbox->addStretch ();
	selectionbuttonbox->addWidget (delbutton);
	selectionbuttonbox->setResizeMode (QLayout::Fixed);
	selectionbuttonbox->setMargin (10);

	QHBoxLayout *hbox = new QHBoxLayout (mainvbox);
	mission_image = new QLabel ("", this);
	mission_image->setFixedSize (300, 200);
	hbox->addWidget (mission_image);
	
	description = new QTextEdit (this);
	description->setMaximumHeight (200);
	description->setSizePolicy (QSizePolicy::Minimum, QSizePolicy::Maximum);
	hbox->addWidget (description);
}

MissionWidget::~MissionWidget() {
	cdp->known_missions.clear ();

	QListViewItem *item = missionlist->firstChild ();
	while (item) {
		cdp->known_missions.append (item->text (2));
		item = item->nextSibling ();
	}

	cdp->saveConfig ();
}

void MissionWidget::addButtonPressed () {
	QStringList list = KFileDialog::getOpenFileNames(cdp->defaultPath (), "*.mission", this, i18n("Select Mission(s) to add"));

	for (QStringList::Iterator it = list.begin(); it != list.end(); ++it) {
		addMission (*it);
	}
}

void MissionWidget::scanButtonPressed () {
	QString path = KFileDialog::getExistingDirectory(cdp->defaultPath (), this, i18n("Select Directory to scan for missions"));

	QStringList list = cdp->scanNewMissions (path);
	for (QStringList::Iterator it = list.begin(); it != list.end(); ++it) {
		addMission (*it);
	}
}

void MissionWidget::delButtonPressed () {
	QListViewItem *item = missionlist->selectedItem ();

	delete item;
	
	selectionChanged ();
}

void MissionWidget::apply () {
	cdp->selected_mission = missionlist->selectedItem ()->text (2);

	cdp->saveConfig ();
}

void MissionWidget::selectionChanged () {
	QListViewItem *item = missionlist->selectedItem ();

	if (item) {
		delbutton->setEnabled (true);

		description->setText (item->text (4));

		mission_image->setPixmap (QPixmap (QImage (item->text (3)).smoothScale (300, 200, QImage::ScaleMin)));

		parent->selectionChanged (MissionSelector::Mission, true);
	} else {
		delbutton->setEnabled (false);

		description->setText (i18n ("No mission selected. Click on a mission above to view a short description. Use the buttons on the top-right to add/remove missions to/from the list."));

		mission_image->clear ();

		parent->selectionChanged (MissionSelector::Mission, false);
	}
}

void MissionWidget::initialize () {
	QStringList list = cdp->getMissionList();
	for (QStringList::Iterator it = list.begin(); it != list.end(); ++it) {
		addMission (*it);
	}

	selectionChanged ();
}

void MissionWidget::addMission (const QString & filename) {
	QListViewItem *item = missionlist->firstChild ();
	while (item) {
		if (item->text (2) == filename) {		// already present
			return;
		} else {
			item = item->nextSibling ();
		}
	}

	Taxipilot::MissionDescription desc = tp->getMissionDescription (filename);

// TODO: print failure notice!
	if (!desc.ok) return;

	item = new QListViewItem (missionlist, desc.title, desc.author, filename, desc.splash_image, desc.abstract);
}


// ########### BEGIN SavesWidget ####################


SavesWidget::SavesWidget (QWidget *parent, MissionSelector *m_sel) : QWidget (parent) {
	SavesWidget::parent = m_sel;

	QGridLayout *mainlayout = new QGridLayout (this);
	QVBoxLayout *mainvbox = new QVBoxLayout (mainlayout);
	mainvbox->setMargin (5);
	QHBoxLayout *selectionhbox = new QHBoxLayout (mainvbox);

	saveslist = new QListView (this);
	saveslist->addColumn (i18n ("Mission"));
	saveslist->addColumn (i18n ("Level"));
	saveslist->addColumn (i18n ("Lives"));
	saveslist->addColumn (i18n ("Fuel"));
	saveslist->addColumn (i18n ("Version"));
	saveslist->setSelectionMode (QListView::Single);
	connect (saveslist, SIGNAL (selectionChanged ()), this, SLOT (selectionChanged ()));
	selectionhbox->addWidget (saveslist);

	QVBoxLayout *selectionbuttonbox = new QVBoxLayout (selectionhbox);
	delbutton = new QPushButton (i18n ("Delete saved game"), this);
	connect (delbutton, SIGNAL (clicked ()), this, SLOT (delButtonPressed ()));
	selectionbuttonbox->addWidget (delbutton);
	selectionbuttonbox->setResizeMode (QLayout::Fixed);
	selectionbuttonbox->setMargin (10);
}

SavesWidget::~SavesWidget () {
	saves->saveList ();
}

void SavesWidget::initialize () {
	saves_pointers = saves->getList ();

	QListViewItem *item;
	for (SaveGames::GameDescription *descr = saves_pointers->first (); descr; descr = saves_pointers->next ()) {
		item = new QListViewItem (saveslist, descr->mission_title, QString::number (descr->level+1), QString::number (descr->lives), QString::number (descr->fuel), descr->mission_version);

		smap.insert (item, descr);
	}
	
	selectionChanged ();
}

void SavesWidget::delButtonPressed () {
	QListViewItem *item = saveslist->selectedItem ();

	saves_pointers->remove (smap[item]);
	smap.remove (item);
	delete item;
	
	selectionChanged ();
}

void SavesWidget::apply () {
	SaveGames::GameDescription *descr = smap[saveslist->selectedItem ()];
	cdp->selected_mission = descr->mission_file;

	cdp->saveConfig ();
	
	cdp->saved_fuel = descr->fuel;
	cdp->saved_level = descr->level;
	cdp->saved_lives = descr->lives;
}

void SavesWidget::selectionChanged () {
	QListViewItem *item = saveslist->selectedItem ();

	if (item) {
		delbutton->setEnabled (true);

		parent->selectionChanged (MissionSelector::Saves, true);
	} else {
		delbutton->setEnabled (false);

		parent->selectionChanged (MissionSelector::Saves, false);
	}
}

#include "missionselector.moc"

