/***************************************************************************
                          preferences.cpp  -  description
                             -------------------
    begin                : Tue Nov 27 2001
    copyright            : (C) 2001, 2002 by Thomas Friedrichsmeier
    email                : Thomas.Friedrichsmeier@ruhr-uni-bochum.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "defines.h"

#include "preferences.h"

#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qbuttongroup.h>
#include <qslider.h>
#include <qlcdnumber.h>
#include <qstring.h>
#include <qlineedit.h>
#include <qfileinfo.h>
#include <qcheckbox.h>
#include <qspinbox.h>
#include <qvariant.h>
#include <qlabel.h>
#include <qframe.h>
#include <qbuttongroup.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>

#include <kfiledialog.h>
#include <klocale.h>

#include "taxipilot.h"
#include "cdp.h"

Preferences::Preferences() : QDialog (tp) {
// do the layout
	setCaption( tr2i18n( "Preferences" ) );
	QGridLayout *mainlayout = new QGridLayout(this);
	QVBoxLayout *mainvbox = new QVBoxLayout(mainlayout);
	mainvbox->setMargin (5);

// tuning/performance section
	QVBoxLayout *tuningvbox = new QVBoxLayout(mainvbox);

	QLabel *label = new QLabel (i18n ("Tuning / Performance settings"), this);
	QFont caption_font = label->font ();
	caption_font.setPointSize(14);
	caption_font.setBold (true);
	label->setFont (caption_font);
	label->setAlignment (QLabel::AlignCenter);
	tuningvbox->addWidget(label);

	QHBoxLayout *fps_cps_layout = new QHBoxLayout(tuningvbox);
	QVBoxLayout *fps_cps_labels_layout = new QVBoxLayout (fps_cps_layout);
	fps_cps_labels_layout->setMargin (5);
	QVBoxLayout *fps_cps_sliders_layout = new QVBoxLayout (fps_cps_layout);
	fps_cps_sliders_layout->setMargin (5);
	QVBoxLayout *fps_cps_numbers_layout = new QVBoxLayout (fps_cps_layout);
	fps_cps_numbers_layout->setMargin (5);

	label = new QLabel (i18n ("Refresh rate (fps)"), this);
	fps_cps_labels_layout->addWidget (label);
	fps_cps_labels_layout->addStretch ();

	label = new QLabel (i18n ("Recalc rate (cps)"), this);
	fps_cps_labels_layout->addWidget (label);

	fps = new QSlider(1, 100, 5, 30, QSlider::Horizontal, this);
	QWhatsThis::add( fps, i18n( "Allows you to adjust the frame rate. Note, that this is a target frame rate, the actual rate may differ from this setting. A higher frame-rater means smoother animations, but also worse performance on slow machines." ) );
	fps_cps_sliders_layout->addWidget (fps);
	fps_cps_sliders_layout->addStretch ();

	base_rate = new QSlider(1, 100, 5, 30, QSlider::Horizontal, this);
	QWhatsThis::add( base_rate, i18n( "Specifies, how often (per second) things like checking for collisions, handling keyboard-input and calculating gravitational forces are done." ) );
	fps_cps_sliders_layout->addWidget (base_rate);

	fpsLCD = new QLCDNumber (this);
	fps_cps_numbers_layout->addWidget (fpsLCD);
	fps_cps_numbers_layout->addStretch ();

	base_rateLCD = new QLCDNumber(this);
	fps_cps_numbers_layout->addWidget (base_rateLCD);

	QHBoxLayout *tuning_checkboxes_layout = new QHBoxLayout(tuningvbox);
	tuning_checkboxes_layout->setMargin (5);
	tuning_checkboxes_layout->addStretch ();

	animate_messages = new QCheckBox(i18n("animate messages"), this);
	QWhatsThis::add(animate_messages, i18n("Whether messages (like \"you crashed\") should be animated, i.e. moved over the screen. Animated messages look nicer, but may sometimes be slow."));
	tuning_checkboxes_layout->addWidget(animate_messages);
	tuning_checkboxes_layout->addStretch ();

	smooth_scroll = new QCheckBox(i18n("smooth scrolling"), this);
	QWhatsThis::add(smooth_scroll, i18n("Scrolling (in oversized levels) can be quite a strain on slow machines. Disabling this option will save resources, but also make scrolling look somewhat jumpy."));
	tuning_checkboxes_layout->addWidget(smooth_scroll);
	tuning_checkboxes_layout->addStretch ();

// division line
	QFrame *line = new QFrame(this);
	line->setSizePolicy(QSizePolicy((QSizePolicy::SizeType)1,(QSizePolicy::SizeType)0, 0, 0, line->sizePolicy().hasHeightForWidth()));
	line->setFrameShadow(QFrame::Sunken);
	line->setFrameShape(QFrame::HLine);
	mainvbox->addWidget( line );

// Sound settings
	QVBoxLayout *soundlayout = new QVBoxLayout(mainvbox);
	soundlayout->addSpacing (10);

	label = new QLabel (i18n("Sound settings"), this);
	label->setFont(caption_font);
	label->setAlignment(QLabel::AlignCenter);
	soundlayout->addWidget (label);

	QHBoxLayout *soundlayout_internal = new QHBoxLayout(soundlayout);
	soundlayout_internal->setMargin (5);
	soundlayout_internal->addStretch ();

	use_sound = new QCheckBox(i18n( "play sounds" ), this);
	soundlayout_internal->addWidget( use_sound );
	soundlayout_internal->addStretch ();

// division line
	line = new QFrame(this);
	line->setFrameShadow(QFrame::Sunken);
	line->setFrameShape(QFrame::HLine);
	mainvbox->addWidget(line);

// ok/cancel-buttons
	QHBoxLayout *okcancellayout = new QHBoxLayout(mainvbox);

	ok_button = new QPushButton(i18n ("Ok"), this);
	okcancellayout->addWidget(ok_button);
	okcancellayout->addStretch ();

	cancel_button = new QPushButton(i18n ("Cancel"), this);
	okcancellayout->addWidget(cancel_button);

// start with reasonable size
    resize(QSize(390, 100).expandedTo(minimumSizeHint()));

/*
    setTabOrder( fps, base_rate );
    setTabOrder( base_rate, ok_button );
    setTabOrder( ok_button, cancel_button ); */

// add the logic
	connect ( ok_button, SIGNAL (clicked ()), SLOT (ok ()) );
	connect ( cancel_button, SIGNAL (clicked ()), SLOT (cancel ()) );
	connect ( base_rate, SIGNAL (valueChanged (int)), SLOT (base_rate_change (int)) );
	connect ( fps, SIGNAL (valueChanged (int)), SLOT (fps_change (int)) );

// get initial values:
	fps->setValue (cdp->pref_fps);
	base_rate->setValue (cdp->pref_cps);
	animate_messages->setChecked (cdp->pref_animate_messages);
	smooth_scroll->setChecked (cdp->pref_scroll_step == 0);
	use_sound->setChecked (cdp->pref_sound);
}

Preferences::~Preferences(){
}

/** Ok-Button pressed. Save changes */
void Preferences::ok(){
	cdp->pref_fps = fps->value ();
	cdp->pref_cps = base_rate->value ();
	cdp->pref_animate_messages = animate_messages->isChecked ();
	cdp->pref_scroll_step = smooth_scroll->isChecked () ? cdp->pref_scroll_step = 0 : 10;

	cdp->pref_sound = use_sound->isChecked ();

	cdp->saveConfig ();

	accept ();
}

/** Cancel-Button pressed. Exit */
void Preferences::cancel(){
	reject ();
}

/** fps-slider got changed. Update displays */
void Preferences::fps_change(int new_val){
	fpsLCD->display (new_val);
	base_rate->setMaxValue ( new_val );
}

/** main_ratio-slider changed. Update displays. */
void Preferences::base_rate_change(int new_val){
	base_rateLCD->display ( new_val );
}

#include "preferences.moc"
