/***************************************************************************
                          settings  -  description
                             -------------------
    begin                : Tue Feb 3 2004
    copyright            : (C) 2004, 2005 by Thomas Friedrichsmeier
    email                : tfry@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "settings.h"

#include <kstandarddirs.h>
#include <kcmdlineargs.h>
#include <kconfig.h>
#include <kapplication.h>

#include "taxipilot.h"

#include "defines.h"

Settings::Settings()
{
	first_start_of_version = false;

	saved_level = -1;

	readOptions ();
	readSettings ();
}

Settings::~Settings()
{
}

void Settings::readOptions () {
	KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
	QString opt = args->getOption("debug-level");
	debug_level = opt.toInt();
	draw_highlights = args->isSet ("highlight");
	opt_thorough_check = args->isSet ("thorough_check");
}

void Settings::readSettings () {
	KConfig *config = kapp->config();

	config->setGroup( "Missions" );
	alt_prefix = prefix = config->readPathEntry ( "defaultMissionDir", KGlobal::dirs()->findResourceDir("game_data", "game_data_tag"));
	selected_mission = config->readPathEntry ( "selected_mission" );
#if KDE_IS_VERSION (3, 1, 3)
	known_missions = config->readPathListEntry ( "known_missions" );
#else
	known_missions = config->readListEntry ( "known_missions" );
#endif
	pref_selectMissionOnStartup = config->readBoolEntry ("select_mission_on_startup", true);

	config->setGroup ( "Tuning" );
	pref_fps = config->readNumEntry( "fps", 60 );
	frame_period = 1000 / pref_fps;
	pref_cps = config->readNumEntry( "cps", 30 );
	pref_animate_messages = config->readBoolEntry ("animate_messages", true);
	pref_scroll_step = config->readNumEntry ("scroll_step", 0);

	config->setGroup ( "Sound" );
	pref_sound = config->readBoolEntry( "sound", true );

	config->setGroup ( "Version" );
	if (config->readEntry ("last_started") != VERSION) {
		first_start_of_version = true;
	}
}

void Settings::saveConfig () {
	KConfig *config = kapp->config();

	if( config ) {
		config->setGroup( "Missions" );
		config->writeEntry ( "defaultMissionDir", prefix );
		config->writeEntry ( "selected_mission", selected_mission );
#if KDE_IS_VERSION (3, 1, 3)
		config->writePathEntry ( "known_missions", known_missions );
#else
		config->writeEntry ( "known_missions", known_missions );
#endif
		config->writeEntry ( "select_mission_on_startup", pref_selectMissionOnStartup );

		config->setGroup ( "Tuning" );
		config->writeEntry( "fps", pref_fps );
		config->writeEntry( "cps", pref_cps );
		config->writeEntry( "animate_messages", pref_animate_messages );
		config->writeEntry ("scroll_step", pref_scroll_step);

		config->setGroup ( "Sound" );
		config->writeEntry ( "sound", pref_sound );

		config->setGroup ( "Version" );
		config->writeEntry ("last_started", VERSION);

		config->sync();
        }

	frame_period = 1000 / pref_fps;
	tp->configChanged ();
}

void Settings::setLastLoaded (const QString & mission){
	selected_mission = mission;
	saveConfig ();
}
