/***************************************************************************
                          splashscreen.h  -  description
                             -------------------
    begin                : Thu Dec 5 2002
    copyright            : (C) 2002 by Thomas Friedrichsmeier
    email                : tfry@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SPLASHSCREEN_H
#define SPLASHSCREEN_H

#include <qdom.h>

#include "timedobject.h"
#include "screen.h"

/**A splash-screen (both at mission-startup and between levels). There are two
	ways of creating a splash-screen: from a very limited set of parameters, or
	from a file, which containts potentially very advanced options.
  *@author Thomas Friedrichsmeier
  */

class SplashScreen : public Screen, public TimedObject {
public: 
/** creates a new splash-screen from a config-file */
	SplashScreen(const QString &config_file, bool show=true);
/** creates a new splash-screen with the given parameters */
	SplashScreen(const QString background_file, const QString &foreground_file, int timeout, bool show=true);
	~SplashScreen();
/** Reads (extended) config for this screen from a file */
	void readSplash (const QDomElement & docElement);
/** Sets background-image for the screen */
	void setBackground (const QString & file);
/** Sets foreground-image for the screen */
	void setForeground (const QString & file);
/** sets time after which splash-screen will disappear automatically (0 is never) */
	void setTimeout (int msecs=0);
/** discards splash (but does not delete it!) */
	void discard ();
/** shows a (discarded) splash */
	void show ();
private:
/** Stores the timeout */
	int timeout; 	
/** Timer-id used for timeout */
	int timeout_id;
/** Initialization stuff */
	void init ();
/** Things to do when done */
	void done ();
protected:
	void timerTick (int id);
};

#endif
